@@ -3,7 +3,7 @@ _exceptions module (imdb package).
 
 This module provides the exception hierarchy used by the imdb package.
 
-Copyright 2004 Davide Alberani <davide.alberani@erlug.linux.it>
+Copyright 2004, 2005 Davide Alberani <da@erlug.linux.it>
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -32,4 +32,7 @@ class IMDbParserError(IMDbError):
     """Exception raised when an error occurred parsing the data."""
     pass
 
+class IMDbNotAvailable(IMDbError):
+    """Exception raised when a required information is not available."""
+    pass
 
