@@ -9,7 +9,7 @@ pages would be:
     plot summary:       http://akas.imdb.com/title/tt0094226/plotsummary
     ...and so on...
 
-Copyright 2004 Davide Alberani <davide.alberani@erlug.linux.it>
+Copyright 2004, 2005 Davide Alberani <da@erlug.linux.it>
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -26,32 +26,32 @@ along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 """
 
-import types
+from urllib import unquote
 
 from imdb.Person import Person
 from imdb.Movie import Movie
 from imdb.utils import analyze_title
+from imdb._exceptions import IMDbParserError
 from utils import ParserBase
 
 
+_stypes = (type(''), type(u''))
+
+
 def strip_amps(theString):
     """Remove '&\s*' at the end of a string.
     It's used to remove '& ' from strings like '(written by) & '.
     """
-    # XXX: check if this function, done with a regexp, will
-    #      save a signifying amount of execution time; I don't think, but...
     i = theString.rfind('&')
     if i != -1:
         if not theString[i+1:] or theString[i+1:].isspace():
             # There's nothing except spaces after the '&'.
-            theString = theString[:i]
+            theString = theString[:i].rstrip()
     return theString
 
 
 def clear_text(theString):
     """Remove separators and spaces in excess."""
-    # XXX: this can be done with a couple of regexp, but I fear it
-    #      will take much more computational time.
     # Squeeze multiple spaces into one.
     theString = ' '.join(theString.split())
     # Remove spaces around the '::' separator.
@@ -71,6 +71,10 @@ class HTMLMovieParser(ParserBase):
         mparser = HTMLMovieParser()
         result = mparser.parse(combined_details_html_string)
     """
+
+    # Do not gather names and titles references.
+    getRefs = 0
+
     def _init(self):
         self.__movie_data = {}
 
@@ -123,7 +127,7 @@ class HTMLMovieParser(ParserBase):
 
     def set_item(self, sect, data):
         """Put a single value (a string, normally) in the dictionary."""
-        if type(data) in (types.StringType, types.UnicodeType):
+        if type(data) in _stypes:
             data = clear_text(data)
         self.__movie_data[clear_text(sect)] = data
 
@@ -165,6 +169,7 @@ class HTMLMovieParser(ParserBase):
         #      with spaces.
         link = self.get_attr_value(attrs, 'href')
         if not link: return
+        olink = link
         link = link.lower()
         if link.startswith('/name'):
             # The following data will be someone's name.
@@ -187,7 +192,7 @@ class HTMLMovieParser(ParserBase):
         elif link.startswith('/glossary'):
             self.__is_cast_crew = 1
             # Get the section name from the link.
-            link = link[12:].replace('_',' ').replace('-',' ')
+            link = link[12:].replace('_', ' ').replace('-', ' ')
             self.__current_section = link
         elif link.startswith('/sections/countries'):
             self.__countries = 1
@@ -199,6 +204,13 @@ class HTMLMovieParser(ParserBase):
             self.__is_mpaa = 1
         elif link.startswith('http://pro.imdb.com'):
             self.__is_movie_status = 0
+        elif link.startswith('/titlebrowse?'):
+            try:
+                d = analyze_title(unquote(olink[13:]))
+                if d.has_key('title'):
+                    self.__movie_data['title'] = d['title']
+            except IMDbParserError:
+                pass
 
     def end_a(self): pass
 
@@ -215,12 +227,21 @@ class HTMLMovieParser(ParserBase):
                 n = n_split[0].strip()
                 del n_split[0]
                 role = ' '.join(n_split)
+                notes = ''
+                ii = role.find('(')
+                if ii != -1:
+                    ei = role.rfind(')')
+                    if ei != -1:
+                        notes = role[ii:ei+1]
+                        role = '%s%s' % (role[:ii], role[ei+1:])
+                        role = role.replace('  ', ' ')
                 # Create a Person object.
                 # XXX: check for self.__cur_nameID?
-                #      maybe it's not a good idea; it's possible that
-                #      a person will be listed without a link?
+                #      maybe it's not a good idea; it's possible for
+                #      a person to be listed without a link?
                 p = Person(name=n, currentRole=role,
                             personID=self.__cur_nameID, accessSystem='http')
+                if notes: p.notes = notes
                 sect = clear_text(self.__current_section)
                 if not self.__movie_data.has_key(sect):
                     self.__movie_data[sect] = []
@@ -250,10 +271,9 @@ class HTMLMovieParser(ParserBase):
             self.__is_company_cred = 0
         elif self.__is_akas and self.__aka_title:
             # XXX: when managing an 'aka title', some transformation
-            #      are required; a complete aka title in the form
+            #      are required; a complete aka title is in the form
             #      of "Aka Title (year) (country1) (country2) [cc]"
-            #      will be stripped of " (year) " if it's equal to
-            #      the movie's year of release; the "Aka Title", the
+            #      The movie's year of release; the "Aka Title", the
             #      countries list and the country code (cc) will be
             #      separated by '::'.
             #      In the example: "Aka Title::(country1) (country2)::[cc]"
@@ -265,8 +285,11 @@ class HTMLMovieParser(ParserBase):
             if year:
                 syear = ' (%s) ' % year
                 if aka.find(syear) != -1:
-                    aka = aka.replace(syear, '::')
+                    aka = aka.replace(syear, '(%s)::' % year)
                 else:
+                    fsti = aka.find(' (')
+                    if fsti != -1:
+                        aka = aka[:fsti] + '::' + aka[fsti+1:]
                     aka = aka.replace(')', ')::')
             ind = aka.rfind(' [')
             if ind != -1:
@@ -282,7 +305,13 @@ class HTMLMovieParser(ParserBase):
         elif self.__is_runtimes and self.__runtimes:
             self.__is_runtimes = 0
             rt = self.__runtimes.replace(' min', '')
-            rt = rt.replace(':', '::')
+            # The "(xy episodes)" note.
+            sep = rt.find(' (')
+            if sep != -1:
+                eep = rt.find(')')
+                if eep > sep:
+                    self.set_item('notes', rt[sep:eep+1])
+                    rt = '%s%s' % (rt[:sep], rt[eep+1:])
             rl = [x.strip() for x in rt.split('/')]
             if rl: self.set_item('runtimes', rl)
 
@@ -326,20 +355,29 @@ class HTMLMovieParser(ParserBase):
                 i = rav.find('/10')
                 if i != -1:
                     rating = rav[:i]
-                    self.set_item('rating', rating)
+                    try:
+                        float(rating)
+                        self.set_item('rating', rating)
+                    except ValueError:
+                        pass
                 i = rav.find('(')
                 if i != -1:
                     votes = rav[i+1:]
                     j = votes.find(' ')
-                    votes = votes[:j]
-                    self.set_item('votes', votes)
+                    votes = votes[:j].replace(',', '')
+                    try:
+                        int(votes)
+                        self.set_item('votes', votes)
+                    except ValueError:
+                        pass
 
     def handle_charref(self, name):
         # This methods is the same found in sgmllib.py, except
-        # that it doesn't call handle_data(), but manage the
+        # that it doesn't call _handle_data(), but manage the
         # entity internally.
+        name = ParserBase.handle_charref(self, name)
         try: n = int(name)
-        except ValueError:
+        except (ValueError, TypeError):
             self.unknown_charref(name)
             return
         if not 0 <= n <= 255:
@@ -373,7 +411,7 @@ class HTMLMovieParser(ParserBase):
         elif self.__is_title:
             self.__title += c
 
-    def handle_data(self, data):
+    def _handle_data(self, data):
         # Manage the plain text part of an HTML document.
         sdata = data.strip()
         sldata = sdata.lower()
@@ -383,12 +421,8 @@ class HTMLMovieParser(ParserBase):
             # Separate the last (...) string; it's here to handle strings
             # like 'screenplay' in "name (screenplay)" in the writing credits.
             if sdata and sdata[0] == '(':
-                data = sdata[1:]
+                data = data.lstrip()
                 self.__name = self.__name.strip() + '::'
-                # Strip everything starting from the last ')'.
-                # XXX: is this useful?  doesn't it cut useful information?
-                ri = data.rfind(')')
-                if ri != -1: data = data[:ri]
             self.__name += data
         elif self.__is_company_cred and self.__current_section:
             # Sometimes company credits are separated by a slash;
@@ -405,7 +439,11 @@ class HTMLMovieParser(ParserBase):
                     self.__company_data += data
                     self.__merge_next = 0
                 else:
-                    if self.__company_data: self.__company_data += '::'
+                    if self.__company_data:
+                        if len(data) > 1:
+                            self.__company_data += '::'
+                        elif data != ' ':
+                            self.__merge_next = 1
                     self.__company_data += data
         elif self.__is_akas and sdata != ':':
             self.__aka_title += data
@@ -469,8 +507,7 @@ class HTMLPlotParser(ParserBase):
 
     def start_p(self, attrs):
         pclass = self.get_attr_value(attrs, 'class')
-        # XXX: if pclass and pclass.lower() ?
-        if pclass == 'plotpar':
+        if pclass and pclass.lower() == 'plotpar':
             self.__is_plot = 1
 
     def end_p(self):
@@ -502,7 +539,7 @@ class HTMLPlotParser(ParserBase):
             self.__plot_writer = ''
             self.__last_plot = ''
             
-    def handle_data(self, data):
+    def _handle_data(self, data):
         # Store text for plots and authors.
         if self.__is_plot:
             self.__plot += data
@@ -520,6 +557,10 @@ class HTMLAwardsParser(ParserBase):
         awparser = HTMLAwardsParser()
         result = awparser.parse(awards_html_string)
     """
+
+    # Do not gather names and titles references.
+    getRefs = 0
+
     def _init(self):
         self.__aw_data = []
         # We're managing awards for a person or a movie?
@@ -612,6 +653,9 @@ class HTMLAwardsParser(ParserBase):
         d = {'year': year, 'result': res, 'award': aw, 'notes': notes,
             'assigner': assign, 'category': cat, 'for': self.__for,
             'to': self.__to, 'with': self.__with}
+        # Remove empty keys.
+        for key in d.keys():
+            if not d[key]: del d[key]
         self.__aw_data.append(d)
         self.__cur_notes = ''
         self.__cur_category = ''
@@ -644,7 +688,7 @@ class HTMLAwardsParser(ParserBase):
         if href.startswith('/Sections/Awards'):
             if self.__in_td:
                 try: year = str(int(href[-4:]))
-                except ValueError: year = None
+                except (ValueError, TypeError): year = None
                 if year:
                     self.__cur_sect = 'year'
                     self.__cur_year = year
@@ -728,7 +772,7 @@ class HTMLAwardsParser(ParserBase):
             self.__cur_role = ''
         self.__is_tn = 0
 
-    def handle_data(self, data):
+    def _handle_data(self, data):
         if self.__is_current_assigner:
             self.__cur_assigner += data
         if not self.__begin_aw or not data or data.isspace() or self.__no:
@@ -766,9 +810,15 @@ class HTMLTaglinesParser(ParserBase):
         tparser = HTMLTaglinesParser()
         result = tparser.parse(taglines_html_string)
     """
+
+    # Do not gather names and titles references.
+    getRefs = 0
+
     def _reset(self):
         """Reset the parser."""
         self.__in_tl = 0
+        self.__in_tlu = 0
+        self.__in_tlu2 = 0
         self.__tl = []
         self.__ctl = ''
 
@@ -779,11 +829,18 @@ class HTMLTaglinesParser(ParserBase):
 
     def start_td(self, attrs):
         # XXX: not good!
-        if self.get_attr_value(attrs, 'width') == "90%":
-            self.__in_tl = 1
+        self.__in_tlu = 1
 
     def end_td(self):
         self.__in_tl = 0
+        self.__in_tlu = 0
+        self.__in_tlu2 = 0
+
+    def start_h1(self, attrs): pass
+
+    def end_h1(self):
+        if self.__in_tlu2:
+            self.__in_tl = 1
 
     def start_p(self, attrs): pass
     
@@ -792,9 +849,11 @@ class HTMLTaglinesParser(ParserBase):
             self.__tl.append(self.__ctl.strip())
             self.__ctl = ''
 
-    def handle_data(self, data):
+    def _handle_data(self, data):
         if self.__in_tl:
             self.__ctl += data
+        elif self.__in_tlu and data.lower().find('taglines for') != -1:
+            self.__in_tlu2 = 1
 
 
 class HTMLKeywordsParser(ParserBase):
@@ -807,6 +866,10 @@ class HTMLKeywordsParser(ParserBase):
         kwparser = HTMLKeywordsParser()
         result = kwparser.parse(keywords_html_string)
     """
+
+    # Do not gather names and titles references.
+    getRefs = 0
+
     def _reset(self):
         """Reset the parser."""
         self.__in_kw = 0
@@ -828,7 +891,7 @@ class HTMLKeywordsParser(ParserBase):
             self.__ckw = ''
             self.__in_kw = 0
 
-    def handle_data(self, data):
+    def _handle_data(self, data):
         if self.__in_kw:
             self.__ckw += data
 
@@ -844,7 +907,6 @@ class HTMLAlternateVersionsParser(ParserBase):
         avparser = HTMLAlternateVersionsParser()
         result = avparser.parse(alternateversions_html_string)
     """
-
     def _init(self):
         self.kind = 'alternate versions'
 
@@ -877,7 +939,7 @@ class HTMLAlternateVersionsParser(ParserBase):
             self.__av.append(self.__cav.strip())
             self.__cav = ''
 
-    def handle_data(self, data):
+    def _handle_data(self, data):
         if self.__in_avd:
             self.__cav += data
 
@@ -892,6 +954,10 @@ class HTMLCrazyCreditsParser(ParserBase):
         ccparser = HTMLCrazyCreditsParser()
         result = ccparser.parse(crazycredits_html_string)
     """
+
+    # Do not gather names and titles references.
+    getRefs = 0
+
     def _reset(self):
         """Reset the parser."""
         self.__in_cc = 0
@@ -907,8 +973,7 @@ class HTMLCrazyCreditsParser(ParserBase):
 
     def start_td(self, attrs):
         # XXX: not good!
-        if self.get_attr_value(attrs, 'width') == "90%":
-            self.__in_cc = 1
+        self.__in_cc = 1
 
     def end_td(self):
         self.__in_cc = 0
@@ -931,11 +996,11 @@ class HTMLCrazyCreditsParser(ParserBase):
     def app(self):
         self.__ccc = self.__ccc.strip()
         if self.__in_cc2 and self.__ccc:
-            self.__cc.append(self.__ccc)
+            self.__cc.append(self.__ccc.replace('\n', ' '))
             self.__ccc = ''
             self.__nrbr = 0
     
-    def handle_data(self, data):
+    def _handle_data(self, data):
         if self.__in_cc2:
             self.__ccc += data
 
@@ -990,7 +1055,7 @@ class HTMLGoofsParser(ParserBase):
             self.__cgo = ''
             self.__cgok = ''
 
-    def handle_data(self, data):
+    def _handle_data(self, data):
         if self.__in_gok:
             self.__cgok += data
         elif self.__in_go2:
@@ -1010,24 +1075,36 @@ class HTMLQuotesParser(ParserBase):
     def _reset(self):
         """Reset the parser."""
         self.__in_quo = 0
+        self.__in_quo2 = 0
         self.__quo = []
         self.__cquo = ''
 
     def get_data(self):
         """Return the dictionary."""
         if not self.__quo: return {}
-        return {'quotes': self.__quo}
+        quo = []
+        for q in self.__quo:
+            if q.endswith('::'): q = q[:-2]
+            quo.append(q)
+        return {'quotes': quo}
 
     def start_td(self, attrs):
         # XXX: not good!
-        if self.get_attr_value(attrs, 'width') == "90%":
-            self.__in_quo = 1
+        self.__in_quo = 1
 
     def end_td(self):
         self.__in_quo = 0
+        self.__in_quo2 = 0
+
+    def start_a(self, attrs):
+        name = self.get_attr_value(attrs, 'name')
+        if name and name.startswith('qt'):
+            self.__in_quo2 = 1
+    
+    def end_a(self): pass
 
     def do_hr(self, attrs):
-        if self.__in_quo and self.__cquo:
+        if self.__in_quo and self.__in_quo2 and self.__cquo:
             self.__cquo = self.__cquo.strip()
             if self.__cquo.endswith('::'):
                 self.__cquo = self.__cquo[:-2]
@@ -1035,16 +1112,16 @@ class HTMLQuotesParser(ParserBase):
             self.__cquo = ''
 
     def do_p(self, attrs):
-        if self.__in_quo:
+        if self.__in_quo and self.__in_quo2:
             self.do_hr([])
             self.__in_quo = 0
 
     def do_br(self, attrs):
-        if self.__in_quo and self.__cquo:
+        if self.__in_quo and self.__in_quo2 and self.__cquo:
             self.__cquo = '%s::' % self.__cquo.strip()
     
-    def handle_data(self, data):
-        if self.__in_quo:
+    def _handle_data(self, data):
+        if self.__in_quo and self.__in_quo2:
             data = data.replace('\n', ' ')
             if self.__cquo.endswith('::'):
                 data = data.lstrip()
@@ -1061,6 +1138,10 @@ class HTMLReleaseinfoParser(ParserBase):
         rdparser = HTMLReleaseinfoParser()
         result = rdparser.parse(releaseinfo_html_string)
     """
+
+    # Do not gather names and titles references.
+    getRefs = 0
+
     def _reset(self):
         """Reset the parser."""
         self.__in_rl = 0
@@ -1101,7 +1182,7 @@ class HTMLReleaseinfoParser(ParserBase):
             self.__rl.append(self.__crl)
             self.__crl = ''
 
-    def handle_data(self, data):
+    def _handle_data(self, data):
         if self.__in_rl2:
             if self.__crl and self.__crl[-1] not in (' ', ':') \
                     and not data.isspace():
@@ -1119,6 +1200,10 @@ class HTMLRatingsParser(ParserBase):
         rparser = HTMLRatingsParser()
         result = rparser.parse(userratings_html_string)
     """
+
+    # Do not gather names and titles references.
+    getRefs = 0
+
     def _reset(self):
         """Reset the parser."""
         self.__in_t = 0
@@ -1143,9 +1228,9 @@ class HTMLRatingsParser(ParserBase):
         """Return the dictionary."""
         data = {}
         if self.__votes:
-            data.update({'number of votes': self.__votes})
+            data['number of votes'] = self.__votes
         if self.__demo:
-            data.update({'demographic': self.__demo})
+            data['demographic'] = self.__demo
         data.update(self.__rank)
         return data
 
@@ -1181,7 +1266,7 @@ class HTMLRatingsParser(ParserBase):
                     n = self.__cur_nr
                     int(c)
                     int(n)
-                    self.__votes.update({c: n})
+                    self.__votes[c] = n
                 except (ValueError, OverflowError): pass
                 self.__cur_nr = ''
                 self.__cur_vote = ''
@@ -1192,7 +1277,7 @@ class HTMLRatingsParser(ParserBase):
                 dv = self.__next_demo_vote
                 float(av)
                 int(dv)
-                self.__demo.update({self.__cur_demo_t: (dv, av)})
+                self.__demo[self.__cur_demo_t] = (dv, av)
             except (ValueError, OverflowError): pass
             self.__cur_demo_av = ''
             self.__next_demo_vote = ''
@@ -1213,7 +1298,7 @@ class HTMLRatingsParser(ParserBase):
     def end_a(self):
         self.__in_demo_t = 0
     
-    def handle_data(self, data):
+    def _handle_data(self, data):
         if self.__in_b and data == 'Rating':
             self.__in_total = 1
         sdata = data.strip()
@@ -1252,6 +1337,10 @@ class HTMLOfficialsitesParser(ParserBase):
         osparser = HTMLOfficialsitesParser()
         result = osparser.parse(officialsites_html_string)
     """
+
+    # Do not gather names and titles references.
+    getRefs = 0
+
     def _reset(self):
         """Reset the parser."""
         self.__in_os = 0
@@ -1301,11 +1390,81 @@ class HTMLOfficialsitesParser(ParserBase):
         
     def end_a(self): pass
     
-    def handle_data(self, data):
+    def _handle_data(self, data):
         if self.__in_os3:
             self.__cos += data
 
 
+class HTMLConnectionParser(ParserBase):
+    """Parser for the "connections" page of a given movie.
+    The page should be provided as a string, as taken from
+    the akas.imdb.com server.  The final result will be a
+    dictionary, with a key for every relevant section.
+
+    Example:
+        connparser = HTMLConnectionParser()
+        result = connparser.parse(connections_html_string)
+    """
+
+    # Do not gather names and titles references.
+    getRefs = 0
+
+    def _reset(self):
+        """Reset the parser."""
+        self.__in_cn = 0
+        self.__in_cnt = 0
+        self.__cn = {}
+        self.__cnt = ''
+        self.__cur_id = ''
+        self.__mtitle = ''
+
+    def get_data(self):
+        """Return the dictionary."""
+        if not self.__cn: return {}
+        return {'connections': self.__cn}
+
+    def start_dt(self, attrs):
+        self.__in_cnt = 1
+        self.__cnt = ''
+
+    def end_dt(self):
+        self.__in_cnt = 0
+
+    def start_dd(self, attrs):
+        self.__in_cn = 1
+
+    def end_dd(self):
+        self.__in_cn = 0
+        self.__cur_id = ''
+
+    def start_a(self, attrs):
+        href = self.get_attr_value(attrs, 'href')
+        if not (self.__in_cn and href and href.startswith('/title')): return
+        tn = self.re_imdbID.findall(href)
+        if tn:
+            self.__cur_id = tn[-1]
+    
+    def end_a(self): pass
+
+    def do_br(self, attrs):
+        sectit = self.__cnt.strip()
+        if self.__in_cn and self.__mtitle and self.__cur_id and sectit:
+            if not self.__cn.has_key(sectit):
+                self.__cn[sectit] = []
+            m = Movie(title=self.__mtitle,
+                        movieID=self.__cur_id,
+                        accessSystem='http')
+            self.__cn[sectit].append(m)
+            self.__mtitle = ''
+            self.__cur_id = ''
+
+    def _handle_data(self, data):
+        if self.__in_cn:
+            self.__mtitle += data
+        elif self.__in_cnt:
+            self.__cnt += data.lower()
+
+
 # The used instances.
 movie_parser = HTMLMovieParser()
 plot_parser = HTMLPlotParser()
@@ -1321,5 +1480,6 @@ quotes_parser = HTMLQuotesParser()
 releasedates_parser = HTMLReleaseinfoParser()
 ratings_parser = HTMLRatingsParser()
 officialsites_parser = HTMLOfficialsitesParser()
+connections_parser = HTMLConnectionParser()
 
 
