@@ -6,9 +6,9 @@ search_movie_parser instance), used to parse the results of a search
 for a given title.
 E.g., for when searching for the title "the passion", the parsed
 page would be:
-    http://akas.imdb.com/find?q=the+passion&tt=on&mx=20
+    http://akas.imdb.com/find?q=the+passion&more=tt
 
-Copyright 2004 Davide Alberani <davide.alberani@erlug.linux.it>
+Copyright 2004, 2005 Davide Alberani <da@erlug.linux.it>
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -29,6 +29,8 @@ from imdb.utils import analyze_title
 from utils import ParserBase
 
 
+# XXX: not sure it's still useful, with the new search system.
+#      Anyway, it's used by the local access system, to get the imdbID.
 class BasicMovieParser(ParserBase):
     """Simply get the title of a movie and the imdbID.
     
@@ -36,6 +38,9 @@ class BasicMovieParser(ParserBase):
     for a direct match (when a search on IMDb results in a single
     movie, the web server sends directly the movie page.
     """
+    # Do not gather names and titles references.
+    getRefs = 0
+
     def _reset(self):
         """Reset the parser."""
         self.__result = []
@@ -54,22 +59,27 @@ class BasicMovieParser(ParserBase):
     def end_title(self):
         self.__reading_page_title = 0
 
-    def start_a(self, attrs):
-        link = self.get_attr_value(attrs, 'href')
-        # XXX: Since July 2004, IMDb has removed the "pageflicker",
-        #      so we've to gather the imdbID from the "submit a comment"
-        #      link.
-        if link and link.lower().startswith('/commentsenter?const='):
-            imdbID = link[21:]
-            if imdbID and self.__page_title:
+    def start_input(self, attrs):
+        # XXX: read the movieID from the "send this page to a friend" form.
+        t = self.get_attr_value(attrs, 'type')
+        if t and t.strip().lower() == 'hidden':
+            n = self.get_attr_value(attrs, 'name')
+            if n and n.strip().lower() == 'arg':
+                val = self.get_attr_value(attrs, 'value') or ''
+                # XXX: use re_imdbIDonly because in the input field
+                #      the movieID is not preceded by 'tt'.
+                nr = self.re_imdbIDonly.findall(val)
+                if not nr: return
+                imdbID = nr[0]
                 title = self.__page_title.strip()
-                res = [(imdbID, analyze_title(title))]
-                self.reset()
-                self.__result = res
+                if imdbID and title:
+                    res = [(imdbID, analyze_title(title, canonical=1))]
+                    self.reset()
+                    self.__result = res
+            
+    def end_input(self): pass
 
-    def end_a(self): pass
-
-    def handle_data(self, data):
+    def _handle_data(self, data):
         if self.__reading_page_title:
             self.__page_title += data
 
@@ -78,6 +88,9 @@ class HTMLSearchMovieParser(ParserBase):
     """Parse the html page that the IMDb web server shows when the
     "new search system" is used.
     """
+    # Do not gather names and titles references.
+    getRefs = 0
+
     def _reset(self):
         """Reset the parser."""
         self.__results = []
@@ -86,6 +99,7 @@ class HTMLSearchMovieParser(ParserBase):
         self.__reading_page_title = 0
         self.__current_imdbID = ''
         self.__current_title = ''
+        self.__no_more = 0
 
     def get_data(self):
         """Return a list of ('imdbID', {title_dict}) tuples."""
@@ -97,10 +111,10 @@ class HTMLSearchMovieParser(ParserBase):
     def end_title(self):
         self.__reading_page_title = 0
 
-    def start_tr(self, attrs):
+    def start_ol(self, attrs):
         self.__begin_list = 1
 
-    def end_tr(self):
+    def end_ol(self):
         self.__begin_list = 0
         self.__is_title = 0
         self.__current_title = ''
@@ -115,21 +129,31 @@ class HTMLSearchMovieParser(ParserBase):
             self.__current_imdbID = nr[0]
             self.__is_title = 1
 
-    def end_a(self):
-        if self.__begin_list and self.__is_title:
+    def end_a(self): pass
+
+    def start_small(self, attrs):
+        self.__no_more = 1
+
+    def end_small(self): pass
+
+    def start_li(self, attrs): pass
+
+    def end_li(self):
+        if self.__begin_list and self.__is_title and self.__current_imdbID:
             # We should have got the title.
             title = self.__current_title.strip()
-            tup = (self.__current_imdbID, analyze_title(title))
+            tup = (self.__current_imdbID, analyze_title(title, canonical=1))
             self.__results.append(tup)
         self.__current_title = ''
         self.__current_imdbID = ''
         self.__is_title = 0
+        self.__no_more = 0
 
-    def handle_data(self, data):
-        if self.__begin_list and self.__is_title:
+    def _handle_data(self, data):
+        if self.__begin_list and self.__is_title and not self.__no_more:
             self.__current_title += data
         elif self.__reading_page_title:
-            if data.strip().lower() != 'imdb title search':
+            if data.lower().find('imdb title search') == -1:
                 # XXX: a direct result!
                 #      Interrupt parsing, and retrieve data using a
                 #      BasicMovieParser object.
@@ -145,8 +169,11 @@ class HTMLSearchMovieParser(ParserBase):
             # XXX: we have to check the plain text part of the HTML
             #      to know when the list of title begins.
             data = data.strip().lower()
-            if data.startswith('exact match') or \
-                    data.startswith('partial match'):
+            if data.find('exact match') != -1 or \
+                    data.find('partial match') != -1 or \
+                    data.find('approx match') != -1 or \
+                    data.find('approximate match') != -1 or \
+                    data.find('popular titles') != -1:
                 self.__begin_list = 1
 
 
